<?php

class WC_Payment_Gateway_BinPaymentByTwintForWoocommerce extends WC_Payment_Gateway
{
	public function __construct()
	{
		$this->id = 'twint';

		$this->icon = '';
		$this->has_fields = true;
		$this->method_title = __('Payment by twint', 'bin-payment-by-invoice-for-woocommerce');
		$this->method_description = __('Customers will be able to pay by twint.', 'bin-payment-by-invoice-for-woocommerce');

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables
		$this->enabled = $this->get_option('enabled');
		$this->title = $this->get_option('title');
		$this->description = $this->get_option('description');
		$this->instructions = $this->get_option('instructions', $this->description);
		$this->default_order_status = $this->get_option('default_order_status', 'on-hold');
		$this->customers_only = $this->get_option('customers_only', 'no');
		$this->customers_completed = $this->get_option('customers_completed', 'no');

		$this->supports = [
			'products',
			'subscriptions',
			'subscription_cancellation',
			'subscription_suspension',
			'subscription_reactivation',
			'subscription_amount_changes',
			'subscription_date_changes',
			'subscription_payment_method_change',
		];

		// Actions
		add_action('woocommerce_update_options_payment_gateways_'.$this->id, [$this, 'process_admin_options']);
		add_action('woocommerce_thankyou_twint', [$this, 'thankyou_page']);
		add_action('woocommerce_scheduled_subscription_payment'.$this->id, [$this, 'process_admin_options']);

		// Customer Emails
		add_action('woocommerce_email_before_order_table', [$this, 'email_instructions'], 10, 3);
	}

	/**
	 * Initialise Gateway Settings Form Fields
	 */
	public function init_form_fields() {

		$this->form_fields = [
			'enabled' => [
				'title' => __('Enable/Disable', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'checkbox',
				'label' => __('Enable Pay by Twint', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => 'no'
			],
			'title' => [
				'title' => _x('Title', 'gateway', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'text',
				'description' => __('The title which the customers will see during checkout.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => __('Pay by Twint', 'bin-payment-by-invoice-for-woocommerce'),
				'desc_tip' => true,
			],
			'description' => [
				'title' => __('Description', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'textarea',
				'description' => __('Payment method description that the customers will see during checkout.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => __('You\'ll receive an invoice after your order. Please transfer the order amount to our twint account.', 'bin-payment-by-invoice-for-woocommerce'),
				'desc_tip' => true,
			],
			'instructions' => [
				'title' => __('Instructions', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'textarea',
				'description' => __('Instructions that will be added to the thank you page and emails.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => '',
				'desc_tip' => true,
			],
			'default_order_status' => [
				'title' => __('Order Status', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'select',
				'description' => __('Choose which order status should be applied after a customer has chosen to pay by twint.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => 'wc-on-hold',
				'options' => wc_get_order_statuses(),
				'desc_tip' => true,
			],
			'customers_only' => [
				'title' => __('Registered customers', 'bin-payment-by-invoice-for-woocommerce'),
				'label' => __('Do only offer pay by Twint to registered/logged in customers.', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'checkbox',
				'description' => __('This will enable Pay by twint to logged in customers only.', 'bin-payment-by-invoice-for-woocommerce'),
				'desc_tip' => true,
				'default' => 'no',
			],
			'customers_completed' => [
				'title' => __('Customer limitation', 'bin-payment-by-invoice-for-woocommerce'),
				'label' => __('Do only offer pay by twint to customers who have at least completed one order.', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'checkbox',
				'default' => 'no',
			],
			'role_only' => [
				'title' => __('Role limitation', 'bin-payment-by-invoice-for-woocommerce'),
				'label' => __('Do only offer pay by twint to selected user roles.', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'checkbox',
				'default' => 'no',
			],
			'selected_role' => [
				'title' => __('Allowed roles', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'multiselect',
				'description' => __('Choose the roles that are allow to pay by twint. To select multiple roles, select while pressing shift.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => 'wc-on-hold',
				'options' => $this->wp_get_user_roles(),
				'desc_tip' => true,
			],
			'min_cart_total' => [
				'title' => __('Min. cart total', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'number',
				'description' => __('The minimum cart total to allow to pay by twint. Leave empty to ignore.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => '',
				'desc_tip' => true,
			],
			'max_cart_total' => [
				'title' => __('Max. cart total', 'bin-payment-by-invoice-for-woocommerce'),
				'type' => 'number',
				'description' => __('The maximum cart total to allow to pay by twint. Leave empty to ignore.', 'bin-payment-by-invoice-for-woocommerce'),
				'default' => '',
				'desc_tip' => true,
			],
		];
	}
	
	private function wp_get_user_roles(){
		global $wp_roles;
		
		$roles = array();
		
		foreach ( $wp_roles->roles as $key=>$value ){
			$array[$key] = $value['name'];
		}
		
		return $array;
	}

	/**
	 * Output for the order received page.
	 */
	public function thankyou_page()
	{
		if($this->instructions)
		{
			echo wpautop(wptexturize($this->instructions));
		}
	}

	/**
	 * Add content to the WC emails.
	 *
	 * @access public
	 *
	 * @param WC_Order $order
	 * @param bool $sent_to_admin
	 * @param bool $plain_text
	 */
	public function email_instructions($order, $sent_to_admin, $plain_text = false)
	{
		$status = str_replace('wc-', '', $this->default_order_status);

		if($this->instructions && !$sent_to_admin && 'twint' === $order->get_payment_method() && $order->has_status($status))
		{
			echo wpautop(wptexturize($this->instructions)).PHP_EOL;
		}
	}

	public function is_available()
	{
		if('yes' != $this->enabled)
		{
			return false;
		}

		if(is_checkout())
		{
			$customers_only_check = true;
			if($this->get_option('customers_only') === 'yes')
			{
				if(is_user_logged_in())
				{
					$customers_only_check = true;
				}
				else
				{
					$customers_only_check = false;
				}
			}
			
			$customers_completed_check = true;
			if($this->get_option('customers_completed') === 'yes')
			{
				if(is_user_logged_in())
				{
					if(WC()->customer->get_is_paying_customer() === true)
					{
						$customers_completed_check = true;
					}
					else
					{
						$customers_completed_check = false;
					}
				}
				else
				{
					$customers_completed_check = false;
				}
			}
			
			$role_only_check = true;
			if($this->get_option('role_only') == 'yes')
			{
				$user = wp_get_current_user();
				$user_roles = ( array ) $user->roles;
				$selected_roles = $this->get_option('selected_role');

				$role_only_check = false;
				foreach($selected_roles as $selected_role)
				{
					if(in_array($selected_role, $user_roles))
					{
						$role_only_check = true;
					}
				}
			}
			
			// cart total
			$cart = WC()->cart;
			$subtotal_tax = $cart->cart_contents_total + $cart->tax_total;
			$shipping_total_tax = $cart->shipping_total + $cart->shipping_tax_total;
			$cart_total_including_tax = $subtotal_tax + $shipping_total_tax;
			
			$min_cart_total_check = true;
			if(is_numeric($this->get_option('min_cart_total')))
			{
				if($cart_total_including_tax >= $this->get_option('min_cart_total'))
				{
					$min_cart_total_check = true;
				}
				else
				{
					$min_cart_total_check = false;
				}
			}
			
			$max_cart_total_check = true;
			if(is_numeric($this->get_option('max_cart_total')))
			{
				if($cart_total_including_tax <= $this->get_option('max_cart_total'))
				{
					$max_cart_total_check = true;
				}
				else
				{
					$max_cart_total_check = false;
				}
			}
			
			if($customers_only_check === true && $customers_completed_check === true && $role_only_check === true && $min_cart_total_check === true && $max_cart_total_check === true)
			{
				return true;
			}

			return false;
		}

		return true;
	}

	/**
	 * Process the payment and return the result
	 *
	 * @param int $order_id
	 *
	 * @return array
	 */
	public function process_payment($order_id)
	{
		$order = wc_get_order($order_id);

		$order->update_status($this->default_order_status);

		// Reduce stock level
		wc_maybe_reduce_stock_levels($order_id);

		// Check if cart instance exists(frontend request only)
		if(WC()->cart)
		{
			// Remove cart
			WC()->cart->empty_cart();
		}

		// Return thankyou redirect
		return [
			'result' => 'success',
			'redirect' => $this->get_return_url($order)
		];
	}
}